@@ -16,19 +16,26 @@ using Gdk;
 
 using FSpot;
 using FSpot.Png;
+using FSpot.UI.Dialog;
+
+using FSpot.Utils;
 
 using Mono.Unix;
 
 namespace FSpot {
 	public class RotateException : ApplicationException {
 		public string path;
+		public bool ReadOnly = false;
 		
 		public string Path {
 			get { return path; }
 		}
+
+		public RotateException (string msg, string path) : this (msg, path, false) {}
 		
-		public RotateException (string msg, string path) : base (msg) {
+		public RotateException (string msg, string path, bool ro) : base (msg) {
 			this.path = path;
+			this.ReadOnly = ro;
 		}
 	}
 
@@ -37,17 +44,15 @@ namespace FSpot {
 		Counterclockwise,
 	}
 
-	public class RotateOperation : Operation {
+	public class RotateOperation : IOperation {
 		IBrowsableItem item;
 		RotateDirection direction;
-		int version_index;
 		bool done;
 
 		public RotateOperation (IBrowsableItem item, RotateDirection direction)
 		{
 			this.item = item;
 			this.direction = direction;
-			version_index = 0;
 			done = false;
 		}
 
@@ -58,18 +63,17 @@ namespace FSpot {
 					     direction == RotateDirection.Clockwise ? JpegUtils.TransformType.Rotate90 
 					     : JpegUtils.TransformType.Rotate270);
 			
-			Unix.Rename (temporary_path, original_path);
+			Utils.Unix.Rename (temporary_path, original_path);
 		}
 
 		private static void RotateOrientation (string original_path, RotateDirection direction)
 		{
 			using (FSpot.ImageFile img = FSpot.ImageFile.Create (original_path)) {
-			
 				if (img is JpegFile) {
 					FSpot.JpegFile jimg = img as FSpot.JpegFile;
 					PixbufOrientation orientation = direction == RotateDirection.Clockwise
-						? PixbufUtils.Rotate90 (img.Orientation)
-						: PixbufUtils.Rotate270 (img.Orientation);
+						? FSpot.Utils.PixbufUtils.Rotate90 (img.Orientation)
+						: FSpot.Utils.PixbufUtils.Rotate270 (img.Orientation);
 				
 					jimg.SetOrientation (orientation);
 					jimg.SaveMetaData (original_path);
@@ -88,15 +92,15 @@ namespace FSpot {
 							supported = true;
 					}
 
-					if (! supported) {
-						throw new RotateException ("Unable to rotate photo type", original_path);
+					if (!supported) {
+						throw new RotateException (Catalog.GetString ("Unable to rotate this type of photo"), original_path);
 					}
 
 					string backup = ImageFile.TempPath (original_path);
 					using (Stream stream = File.Open (backup, FileMode.Truncate, FileAccess.Write)) {
 						using (Pixbuf pixbuf = img.Load ()) {
 							PixbufOrientation fake = (direction == RotateDirection.Clockwise) ? PixbufOrientation.RightTop : PixbufOrientation.LeftBottom;
-							using (Pixbuf rotated = PixbufUtils.TransformOrientation (pixbuf, fake)) {
+							using (Pixbuf rotated = FSpot.Utils.PixbufUtils.TransformOrientation (pixbuf, fake)) {
 								img.Save (rotated, stream);
 							}
 						}
@@ -104,7 +108,7 @@ namespace FSpot {
 					File.Copy (backup, original_path, true);
 					File.Delete (backup);
 				} else {
-					throw new RotateException ("Unable to rotate photo type", original_path);
+					throw new RotateException (Catalog.GetString ("Unable to rotate this type of photo"), original_path);
 				}
 			}
 		}
@@ -114,28 +118,22 @@ namespace FSpot {
 			RotateOrientation (original_path, dir);
 		}
 		
-		public override bool Step () {
+		public bool Step () {
 			string original_path;
 
 			if (done)
 				return false;
 
-			if (item is Photo) {
-				Photo p = (Photo) item;
+ 			original_path = item.DefaultVersionUri.LocalPath;
+ 			done = true;
 
-				original_path = p.GetVersionPath (p.VersionIds [version_index++]);
-				done = (version_index >= p.VersionIds.Length);
-			} else {
-				original_path = item.DefaultVersionUri.LocalPath;
-				done = true;
+			if ((File.GetAttributes(original_path) & FileAttributes.ReadOnly) == FileAttributes.ReadOnly) {
+				throw new RotateException (Catalog.GetString ("Unable to rotate readonly file"), original_path, true);
 			}
 
-			if ((File.GetAttributes(original_path) & FileAttributes.ReadOnly) == FileAttributes.ReadOnly)
-				throw new RotateException ("Unable to rotate readonly file", original_path);
-
 			Rotate (original_path, direction);
 
-			Gdk.Pixbuf thumb = FSpot.ThumbnailGenerator.Create (original_path);
+			Gdk.Pixbuf thumb = FSpot.ThumbnailGenerator.Create (UriUtils.PathToFileUri (original_path));
 			if (thumb != null)
 				thumb.Dispose ();
 		
@@ -143,7 +141,7 @@ namespace FSpot {
 		}
 	}
 
-	public class RotateMultiple : Operation {
+	public class RotateMultiple : IOperation {
 		RotateDirection direction;
 		IBrowsableItem [] items;
 		int index;
@@ -164,7 +162,7 @@ namespace FSpot {
 			index = 0;
 		}
 		
-		public override bool Step ()
+		public bool Step ()
 		{
 			if (index >= items.Length)  
 				return false;
@@ -204,23 +202,30 @@ public class RotateCommand {
 	        RotateMultiple op = new RotateMultiple (items, direction);
 		int readonly_count = 0;
 		bool done = false;
-		int index = -1;
+		int index = 0;
 
 		while (!done) {
-			if (progress_dialog != null && index != op.Index) 
+			if (progress_dialog != null && op.Index != -1 && index < items.Length) 
 				if (progress_dialog.Update (String.Format (Catalog.GetString ("Rotating photo \"{0}\""), op.Items [op.Index].Name)))
 					break;
 
 			try {
 				done = !op.Step ();
 			} catch (RotateException re) {
-				if (re.Message == "Unable to rotate photo type")
-					RunTypeError (re);
+				if (!re.ReadOnly)
+					RunGenericError (re, re.Path, re.Message);
 				else
-					readonly_count ++;
+					readonly_count++;
+			} catch (GLib.GException) {
+				readonly_count++;
+			} catch (DirectoryNotFoundException e) {
+				RunGenericError (e, op.Items [op.Index].DefaultVersionUri.LocalPath, Catalog.GetString ("Directory not found"));
+			} catch (FileNotFoundException e) {
+				RunGenericError (e, op.Items [op.Index].DefaultVersionUri.LocalPath, Catalog.GetString ("File not found"));
 			} catch (Exception e) {
 				RunGenericError (e, op.Items [op.Index].DefaultVersionUri.LocalPath);
 			}
+			index ++;
 		}
 		
 		if (progress_dialog != null)
@@ -234,14 +239,15 @@ public class RotateCommand {
 
 	private void RunReadonlyError (int readonly_count)
 	{
-		string notice = Catalog.GetPluralString ("Unable to rotate photo",  
-								    "Unable to rotate {0} photos",  
-								    readonly_count);
-		
-		string desc = Catalog.GetPluralString ("The photo could not be rotated because it is on a read only file system or " + 
-								  "media such as a CDROM.  Please check the permissions and try again",  
-								  "{0} photos could not be rotated because they are on a read only file system " + 
-								  "or media such as a CDROM.  Please check the permissions and try again",  readonly_count);
+		string notice = Catalog.GetPluralString ("Unable to rotate photo", "Unable to rotate {0} photos", readonly_count);
+		string desc = Catalog.GetPluralString (
+			"The photo could not be rotated because it is on a read only file system or media such as a CDROM.  Please check the permissions and try again.",
+			"{0} photos could not be rotated because they are on a read only file system or media such as a CDROM.  Please check the permissions and try again.",
+			readonly_count
+		);
+
+		notice = String.Format (notice, readonly_count);
+		desc = String.Format (desc, readonly_count);
 		
 		HigMessageDialog md = new HigMessageDialog (parent_window, 
 							    DialogFlags.DestroyWithParent,
@@ -253,15 +259,17 @@ public class RotateCommand {
 		md.Destroy();
 	}
 	
-	private void RunTypeError (RotateException re)
-	{
-		RunGenericError (re, re.Path);
-	}
-
+	// FIXME shouldn't need this method, should catch all exceptions explicitly
+	// so can present translated error messages.
 	private void RunGenericError (System.Exception e, string path)
+	{
+		RunGenericError (e, path, e.Message);
+	}
+
+	private void RunGenericError (System.Exception e, string path, string msg)
 	{
 		string longmsg = String.Format (Catalog.GetString ("Received error \"{0}\" while attempting to rotate {1}"),
-						e.Message, System.IO.Path.GetFileName (path));
+						msg, System.IO.Path.GetFileName (path));
 
 		HigMessageDialog md = new HigMessageDialog (parent_window, DialogFlags.DestroyWithParent,
 							    MessageType.Warning, ButtonsType.Ok,
