@@ -84,6 +84,7 @@ namespace FSpot {
 		GdkGlx.Context glx;
 		float angle = 0.0f;
 		Animator flip;
+		bool running = false;
 
 		public TextureDisplay (BrowsablePointer item)
 		{
@@ -91,12 +92,18 @@ namespace FSpot {
 			DoubleBuffered = false;
 			AppPaintable = true;
 			CanFocus = true;
-
 			item.Changed += HandleItemChanged;
+
 			flip = new Animator (6000, 6000, delegate { flip.Start (); item.MoveNext (true); }); 
 			flip.RunWhenStarted = false;
 		}
 
+		public override void Dispose ()
+		{
+			if (glx != null)
+				glx.Destroy ();
+		}
+
 		GlTransition [] transitions = new GlTransition []
 			{
 				new GlTransition.Dissolve (),
@@ -134,6 +141,18 @@ namespace FSpot {
 			return true;
 		}
 		
+		public void Start ()
+		{
+			running = true;
+			flip.Start ();
+		}
+
+		public void Stop ()
+		{
+			running = false;
+			flip.Stop ();	
+		}
+
 		private Texture CreateTexture ()
 		{
 			if (glx == null || GdkWindow == null)
@@ -143,9 +162,13 @@ namespace FSpot {
 
 			Texture tex;
 			try {
-				ImageFile img = ImageFile.Create (item.Current.DefaultVersionUri);
-				using (Gdk.Pixbuf pixbuf = img.Load ()) {
-				        tex = new Texture (pixbuf);
+				using (ImageFile img = ImageFile.Create (item.Current.DefaultVersionUri)) {
+					using (Gdk.Pixbuf pixbuf = img.Load ()) {
+						Cms.Profile screen_profile;
+						if (FSpot.ColorManagement.Profiles.TryGetValue (Preferences.Get<string> (Preferences.COLOR_MANAGEMENT_DISPLAY_PROFILE), out screen_profile)) 
+							FSpot.ColorManagement.ApplyProfile (pixbuf, img.GetProfile (), screen_profile);
+						tex = new Texture (pixbuf);
+					}
 				}
 			} catch (Exception) {
 				tex = new Texture (PixbufUtils.ErrorPixbuf);
@@ -154,28 +177,20 @@ namespace FSpot {
 		}
 
 		Animator animator;
-		private void HandleItemChanged (BrowsablePointer p, BrowsablePointerChangedArgs args)
+		private void HandleItemChanged (object sender, BrowsablePointerChangedEventArgs args)
 		{
 			Animator = new Animator (3000, 20, HandleTick);
 
 			if (glx == null)
 				return;
 
-			//Console.WriteLine ("Begin previous = {0} texture = {1}", 
-			//		   previous != null ? previous.Id.ToString () : "null", 
-			//		   next != null ? next.Id.ToString () : "null");
-
 			if (!item.IsValid || item.Collection.Count < 0)
 				return;
 
-			Next = null;
+			//Next = null;
 			PreloadNext ();
 
-			//Console.WriteLine ("End previous = {0} texture = {1}", 
-			//		   previous != null ? previous.Id.ToString () : "null", 
-			//		   next != null ? next.Id.ToString () : "null");
-
-			if (IsRealized)
+			if (running)
 				Animator.Start ();
 		}
 
@@ -305,7 +320,7 @@ namespace FSpot {
 				return previous;
 			}
 			set {
-				if (previous != next)
+				if (previous != next && previous != null)
 					previous.Dispose ();
 
 				previous = value;
@@ -315,7 +330,8 @@ namespace FSpot {
 		Texture next;
 		public Texture Next {
 			get {
-				PreloadNext ();
+				if (next == null)
+					PreloadNext ();
 				return next;
 			}
 			set {
@@ -327,9 +343,7 @@ namespace FSpot {
 
 		private void PreloadNext ()
 		{
-			if (next == null)
-				next = CreateTexture ();
-
+			Next = CreateTexture ();
 		}
 
 		GlTransition transition;
@@ -412,6 +426,10 @@ namespace FSpot {
 			//Gl.glTexEnvi (Gl.GL_TEXTURE_ENV, Gl.GL_TEXTURE_ENV_MODE, Gl.GL_MODULATE);
 			//Gl.glTexParameteri (Gl.GL_TEXTURE_RECTANGLE_ARB, Gl.GL_TEXTURE_MIN_FILTER, Gl.GL_LINEAR);
 			//Gl.glTexParameteri (Gl.GL_TEXTURE_RECTANGLE_ARB, Gl.GL_TEXTURE_MAG_FILTER, Gl.GL_LINEAR);
+			float [] black = new float [] { 0f, 0f, 0f, 1f};			       
+			Gl.glTexParameterfv (Gl.GL_TEXTURE_RECTANGLE_ARB, Gl.GL_TEXTURE_BORDER_COLOR, black);
+			Gl.glTexParameteri (Gl.GL_TEXTURE_RECTANGLE_ARB, Gl.GL_TEXTURE_WRAP_S, Gl.GL_CLAMP_TO_BORDER_ARB);
+			Gl.glTexParameteri (Gl.GL_TEXTURE_RECTANGLE_ARB, Gl.GL_TEXTURE_WRAP_T, Gl.GL_CLAMP_TO_BORDER_ARB);
 			
 			// Viewing transformation
 			Gl.glClear (Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
